#!/bin/python

import xbmc
import xbmcgui
import xbmcplugin
import xbmcaddon
import sys
import urllib
import requests
import os
import re

from resources.lib.artistinfo import ArtistInfo
import thread

# Written by:	Phantom Raspberry Blower
# Date:		21-02-2019
# Description:	Addon for listening to PRB Radio live broadcasts

# Get addon details
__addon_id__ = 'plugin.audio.prb-radio'
__addon__ = xbmcaddon.Addon(id=__addon_id__)
__addonname__ = __addon__.getAddonInfo('name')
__icon__ = __addon__.getAddonInfo('icon')
__fanart__ = __addon__.getAddonInfo('fanart')
__author__ = "Phantom Raspberry Blower"
__url__ = sys.argv[0]
__handle__ = int(sys.argv[1])

# Get localized language words
__language__ = __addon__.getLocalizedString
_prb_radio = __language__(30101)
_artist_info = __language__(30102)
_unable_to_download_artist_info = __language__(30103)
_no_artist_name_present = __language__(30104)
_no_audio_stream = __language__(30105)
_settings = __language__(30106)
_prb_desc = __language__(30107)
_artist_info_desc = __language__(30108)
_settings_desc = __language__(30109)
_prb_radio_not_broadcasting = __language__(30110)
_on_air = __language__(30111)
_prb_radio_off_air = __language__(30112)

# Get addon user settings
_hide_artist_info = __addon__.getSetting('hide_artist_info')
_hide_artist_artwork = __addon__.getSetting('hide_artist_artwork')

# Define local variables
playable_url = ''
base_url = 'http://dir.xiph.org/search?search=PRB+Leicester'
image_path = xbmc.translatePath(os.path.join('special://home/addons/',
	                            __addon_id__ + '/resources/media/'))
g_default_image = os.path.join(image_path, 'prb_radio_fanart.jpg')
links_info = {_prb_radio:
              {'thumbs': os.path.join(image_path, 
                                      'prb_radio.png'),
               'fanart': os.path.join(image_path,
                                      'prb_radio_fanart.jpg'),
               'desc': _prb_desc
               },
              _artist_info:
              {'thumbs': os.path.join(image_path,
                                      'artist_info.png'),
               'fanart': os.path.join(image_path,
                                      'artist_info_fanart.jpg'),
               'desc': _artist_info_desc
               },
              _settings:
              {'thumbs': os.path.join(image_path,
                                      'settings.png'),
               'fanart': os.path.join(image_path,
                                      'settings_fanart.jpg'),
               'desc': _settings_desc
               }
              }


def regex_from_to(text, from_string, to_string, excluding=True):
    """
    Return text between two strings.
    """
    if excluding:
        r = re.search("(?i)" + from_string +
                      "([\S\s]+?)" +
                      to_string, text).group(1)
    else:
        r = re.search("(?i)(" +
                      from_string +
                      "[\S\s]+?" +
                      to_string +
                      ")", text).group(1)
    return r


def get_stream_link():
    """
    Return list of live links.
    """
    link = ''
    title = ''
    resp = requests.get(base_url)
    content = resp.text
    content = content.replace('\n', '').replace('\t', '')
    streams = re.compile('<tr class="(.+?)">(.+?)</tr>').findall(content)
    try:
        for row_id, stream in streams:
            match = re.compile('<p class="stream-name"><span class="name">' \
                               '<a href="(.+?)"(.+?)>(.+?)</a(.+?)>M3U</a>' \
                               '(.+?)<a href="(.+?).xspf"').findall(stream)
            for search, junk, name, junk2, junk3, xspf in match:
                if name == 'PRB Radio':
                	link = 'http://dir.xiph.org' + xspf + '.xspf'
        resp = requests.get(link)
        content = resp.text
        title = regex_from_to(content, '<title>', '</title>')
    except:
        pass
    if title == 'PRB Radio':
        location = regex_from_to(content, '<location>', '</location>')
        return location
    else:
        return 0


def addDir(name, url, mode, icon, fanart, desc, IsPlayable=False):
    """
    Display a list of links
    """
    u = (sys.argv[0] + '?url=' + urllib.quote_plus(url) + '&mode=' + str(mode))
    ok = True
    liz = xbmcgui.ListItem(name)
    # Set fanart and thumb images for the list item.
    if not fanart:
        # Set fanrt to default image
        fanart = __fanart__
    if not icon:
        # Set icon to default image
        icon = __icon__

    if IsPlayable is True:
        liz.setProperty('IsPlayable', 'true')

    liz.setArt({'fanart': fanart, 'thumb': icon})
    # Set additional info for the list item.
    liz.setInfo(type='music',
                infoLabels={'title': name,
                            'artist': name,
                            'comment': desc,
                            'genre': 'Blues, Gospel, Jazz, Soul, Reggae, Funk, Hip-Hop',
                            'year': 2019,
                            'mediatype': 'album'})

    ok = xbmcplugin.addDirectoryItem(handle=__handle__,
                                     url=u,
                                     listitem=liz,
                                     isFolder=False)
    return ok


def main_menu():
    """
    Displays main menu.
    """
    playable_url = get_stream_link()
    if playable_url == 0:
        message(_prb_radio_not_broadcasting, _prb_radio_off_air)
        return 0
    # Live stream link
    addDir(_prb_radio,
           playable_url,
           1,
           links_info[_prb_radio]['thumbs'],
           links_info[_prb_radio]['fanart'],
           links_info[_prb_radio]['desc'],
           IsPlayable=True)
    if _hide_artist_info != 'true':
        # Artist information link
        addDir(_artist_info,
               'XBMC.RunPlugin({0}?url=artistinfo&mode=2)',
               2,
               links_info[_artist_info]['thumbs'],
               links_info[_artist_info]['fanart'],
               links_info[_artist_info]['desc'])
    # Settings link
    addDir(_settings,
           'XBMC.RunPlugin({0}?url=settings&mode=3)',
           3,
           links_info[_settings]['thumbs'],
           links_info[_settings]['fanart'],
           links_info[_settings]['desc'])
    xbmcplugin.endOfDirectory(__handle__)


def get_audio(url):
    """
    Start audio stream
    """
    play_item = xbmcgui.ListItem(path=url)
    xbmcplugin.setResolvedUrl(__handle__, True, listitem=play_item)
    # Wait for audio stream to start
    while not xbmc.Player().isPlayingAudio():
        xbmc.sleep(3)
    # Start thread to display current artist image
    thread.start_new_thread(show_artist_image,())


def get_artist_name():
    """
    Parse artist name from stream title
    """
    # Check audio atream is playing 
    if xbmc.Player().isPlayingAudio():
        # Get title of audio stream
        title = xbmc.Player().getMusicInfoTag().getTitle()
        if len(title) > 0:
            # Split artist name from stream title
            items = title.split(' - ')
            artist_name = items[0]
            if len(artist_name) > 1:
                return artist_name
        else:
            # No artist name present
            notification(_no_artist_name_present, _artist_info, __icon__, 5000)
    else:
        # No audio stream playing
        message(_no_audio_stream, _artist_info)
    return None


def show_artist_details(artistname):
    """
    Shows artist information ie. image, genre, description etc.
    """
    artist_info = ArtistInfo(artistname)
    if artist_info.artist_id is not None:
        home = xbmc.translatePath('special://home')
        if xbmc.getInfoLabel('System.ProfileName') != 'Master user':
            you = xbmc.getInfoLabel('System.ProfileName')
        elif (xbmc.getCondVisibility('System.Platform.Windows') is True or
              xbmc.getCondVisibility('System.Platform.OSX') is True):
            if 'Users\\' in home:
                proyou = str(home).split('Users\\')
                preyou = str(proyou[1]).split('\\')
                you = preyou[0]
            else:
                you = 'You'
        else:
            you = 'You'
        # Open custom artist information dialog
        win = xbmcgui.WindowXMLDialog('artist-info.xml',
                                      __addon__.getAddonInfo('path'))
        # Set artist information properties within xml dialog for example
        # <label fallback="416">$INFO[Window.Property(HeadingLabel)]</label>
        win.setProperty('HeadingLabel', artistname)
        win.setProperty('ArtistImage', artist_info.fanart)
        win.setProperty('ArtistStyle', artist_info.style)
        win.setProperty('ArtistGenre', artist_info.genre)
        win.setProperty('ArtistName', artist_info.artist_name)
        win.setProperty('ArtistFormed',
                        str(artist_info.year_formed).replace('None', ''))
        win.setProperty('ArtistBorn',
                        str(artist_info.year_born).replace('None', ''))
        win.setProperty('ArtistDied',
                        str(artist_info.year_died).replace('None', ''))
        win.setProperty('ArtistGender', artist_info.gender)
        win.setProperty('ArtistCountry',
                        str(artist_info.country).replace('None', '') + ' ' +
                        str(artist_info.country_code).replace('None', ''))
        win.setProperty('ArtistWebsite', artist_info.website)
        win.setProperty('Description', artist_info.biography_en)
        win.doModal()
        del win
    else:
        # Unable to download artist information
        notification(_artist_info,
                     _unable_to_download_artist_info,
                     xbmcgui.NOTIFICATION_INFO, 5000)


def get_current_artist_image():
    """
    Discover artist name, download image and return the image path
    if no image can be found return the default image instead
    """
    # Check if user selected to hide artist image
    if _hide_artist_artwork != 'true':
        # Find the current artist name
        artist_name = get_artist_name()
        if artist_name is not None:
            if len(artist_name) > 1:
                # Fetch the artists information
                artist_info = ArtistInfo(artist_name)
                # Check for artist image
                if artist_info.fanart is not None:
                    if len(artist_info.fanart) > 1:
                        return artist_info.fanart
    return g_default_image


def show_artist_image():
    """
    Shows current artist as background image.
    """
    # Open custom music visualization dialog
    window = xbmcgui.WindowXMLDialog('music-visualization.xml',
                                     __addon__.getAddonInfo('path'))
    window.show()
    previous_title = ''
    # While steaming audio check for changes in title 
    while (not xbmc.Monitor().abortRequested() and
           xbmc.Player().isPlayingAudio()):
        my_title = xbmc.Player().getMusicInfoTag().getTitle()
        # Poll for changes in title 
        if my_title != previous_title:
            # Get current artist image
            artist_image = get_current_artist_image()
            # Set artist image property within xml dialog
            window.setProperty('ArtistFanart',
                               artist_image)
            previous_title = my_title
        xbmc.sleep(1000)
    window.close()
    del window


def get_params():
    """
    Parse the paramters sent to the application
    """
    param = []
    paramstring = sys.argv[2]
    if len(paramstring) >= 2:
        params = sys.argv[2]
        cleanedparams = params.replace('?', '')
        if (params[len(params)-1] == '/'):
            params = params[0:len(params)-2]
        pairsofparams = cleanedparams.split('&')
        param = {}
        for i in range(len(pairsofparams)):
            splitparams = {}
            splitparams = pairsofparams[i].split('=')
            if (len(splitparams)) == 2:
                param[splitparams[0]] = splitparams[1]
    return param


def notification(heading, message, icon, duration):
    # Display notification to user
    dialog = xbmcgui.Dialog()
    dialog.notification(heading, message, icon, duration)


def message(message, title):
    # Display message to user
    dialog = xbmcgui.Dialog()
    dialog.ok(title, message)


# Define local variables
params = get_params()
url = None
mode = None

# Parse the url, name, mode, icon and fanart parameters
try:
    url = urllib.unquote_plus(params['url'])
except:
    pass
try:
    mode = int(params['mode'])
except:
    pass

# Clear the playlist 
xbmc.PlayList(0).clear()
# Route the request based upon the mode number
if mode is None or url is None or len(url) < 1: # Show main menu
    main_menu()
elif mode == 1: # Start audio stream
    # Run as fullscreen otherwise showing current artist description locks up!!??
    xbmc.executebuiltin('Action(Fullscreen)')
    # Start audio stream
    get_audio(url)
    # Close fullscreen (see above)
    xbmc.executebuiltin('Action(Back)')
elif mode == 2: # Show current artist description
    # Parse artist name
    artist_name = get_artist_name()
    if artist_name is not False:
        if artist_name is not None:
            show_artist_details(artist_name)
            thread.start_new_thread(show_artist_image,())
elif mode == 3: # Show settings
    __addon__.openSettings()
    xbmc.executebuiltin("Container.Refresh")

